// Wyatt Studio Copyright (c) 2023

// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/widgets.dart';
import 'package:wyatt_component_copy_with_extension/wyatt_component_copy_with_extension.dart';
import 'package:wyatt_ui_components/wyatt_ui_components.dart';
import 'package:wyatt_ui_kit/src/components/gradients/gradient_text_style.dart';

part 'gradient_text.g.dart';

@ComponentCopyWithExtension()
class GradientText extends GradientTextComponent with $GradientTextCWMixin {
  const GradientText({
    required super.data,
    required super.gradientColors,
    super.style,
    super.key,
    super.strutStyle,
    super.textAlign,
    super.textDirection,
    super.locale,
    super.softWrap,
    super.overflow,
    super.textScaleFactor,
    super.maxLines,
    super.semanticsLabel,
    super.textWidthBasis,
    super.textHeightBehavior,
    super.selectionColor,
  });

  /// Creates a [GradientText] from a [TextWrapper].
  ///
  /// You can pass a gradient in case the [TextWrapper] doesn't have one.
  /// You can also pass a style in case the [TextWrapper] doesn't have one.
  factory GradientText.fromWrapper(
    TextWrapper wrapper, {
    MultiColor? gradientColors,
    TextStyle? style,
  }) =>
      GradientText(
        data: wrapper.data,
        gradientColors:
            wrapper.gradientColors ?? gradientColors ?? const MultiColor([]),
        style: wrapper.style ?? style,
        textAlign: wrapper.textAlign,
        textDirection: wrapper.textDirection,
        softWrap: wrapper.softWrap,
        overflow: wrapper.overflow,
        maxLines: wrapper.maxLines,
        selectionColor: wrapper.selectionColor,
      );

  factory GradientText.fromGradientStyle(
    String data, {
    required GradientTextStyle style,
  }) =>
      GradientText(
        data: data,
        gradientColors: style.gradientColors ?? const MultiColor([]),
        style: style,
      );

  @override
  Widget build(BuildContext context) {
    if (super.style is GradientTextStyle) {
      // If the style is a gradient style, create a gradient text from it.
      final Gradient? linearGradient = GradientHelper.linearFromMultiColor(
        (super.style as GradientTextStyle?)?.gradientColors ??
            const MultiColor([]),
      );
      if (linearGradient != null) {
        return ShaderMask(
          blendMode: BlendMode.srcIn,
          shaderCallback: (bounds) => linearGradient.createShader(
            Rect.fromLTWH(0, 0, bounds.width, bounds.height),
          ),
          child: super.build(context),
        );
      }
    }

    final Gradient? linearGradient =
        GradientHelper.linearFromMultiColor(gradientColors);
    if (linearGradient != null) {
      return ShaderMask(
        blendMode: BlendMode.srcIn,
        shaderCallback: (bounds) => linearGradient.createShader(
          Rect.fromLTWH(0, 0, bounds.width, bounds.height),
        ),
        child: super.build(context),
      );
    }

    if (gradientColors.colors.isNotEmpty) {
      return super
          .copyWith
          .call(style: style?.copyWith(color: gradientColors.color))
          .build(context);
    }

    // If there is no gradient and no color, just build the text.
    return super.build(context);
  }
}
